package com.muyu.service.serviceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.muyu.BasicEngine;
import com.muyu.abstracts.DataValue;
import com.muyu.common.core.domain.Result;
import com.muyu.domain.EngineMaintenance;
import com.muyu.domain.EngineVersion;
import com.muyu.domain.constants.PageResult;
import com.muyu.mapper.EngineMapper;
import com.muyu.req.EngineMaintenanceQueryReq;
import com.muyu.req.EngineVersionListResp;
import com.muyu.service.EngIneService;
import com.muyu.service.EngineVersionService;
import com.muyu.test.ExternalClassLoader;
import jakarta.validation.Path;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.nio.file.Paths;
import java.util.List;

import static com.muyu.test.RuleTest.engineWorkSourcePath;

/**
 * @Author：qdm
 * @Package：com.muyu.service.serviceImpl
 * @Project：cloud-etl-engine
 * @name：EngIneServiceImpl
 * @Date：2024/8/22 15:37
 */
@Service
public class EngIneServiceImpl extends ServiceImpl<EngineMapper, EngineMaintenance> implements EngIneService {


    @Autowired
    EngineMapper engineMapper;

    @Autowired
    private EngineVersionService engineConfigService;

    @Override
    public Object measurementList(EngineVersion engineVersion) {
//        EngineVersion dataValue = engineVersion;
//        String versionClass = engineVersion.getVersionClass();
//        CompileService.test(engineVersion.getVersionCode(), engineVersion.getVersionClass());
//        try {
//            //外部类文件路径
//            String externalClassFilePath = engineWorkSourcePath + "/" + objectPath + "/" + versionClass + ".class";
//            Path classFilePath = Paths.get(externalClassFilePath);
//            //获取外部类所在的目录
//            String externalClassDir = externalClassFilePath.substring(0, externalClassFilePath.lastIndexOf('/'));
//            URL[] urls = null;
//            urls = new URL[]{new File(externalClassDir).toURI().toURL()};
//            //创建自定义类加载
//            ExternalClassLoader externalClassLoader = new ExternalClassLoader(urls);
//
//            //加载类
//            Class<?> clazz = externalClassLoader.loadClassFromPath(classFilePath, packagePath + "." + versionClass);
//            //创建类的实例
//            Object instance = clazz.getDeclaredConstructor().newInstance();
//
//            engineMap.put(versionClass, (BasicEngine<DataValue>) instance);
//        } catch (NoSuchMethodException | InstantiationException | IllegalAccessException | InvocationTargetException |
//                 IOException e) {
//            throw new RuntimeException(e);
//        }
//
//        BasicEngine<DataValue> engine = engineMap.get(versionClass);
//        engine.set(dataValue);
//        engine.execution();
//        return Result.success();
        return null;
    }

    @Override
    public List<EngineMaintenance> list() {
        LambdaQueryWrapper<EngineMaintenance> queryWrapper = new LambdaQueryWrapper<>();
        return list(queryWrapper);
    }

    @Override
    public Result delete(Integer id) {
        Integer res = engineMapper.deleteById(id);
        if (res > 0) {
            return Result.success("成功");
        }
        return Result.error("失败");
    }

    @Override
    public Result<EngineMaintenance> updateMsg(EngineMaintenance engineMaintenance) {
        Integer res = engineMapper.updateById(engineMaintenance);
        if (res > 0) {
            return Result.success();
        }
        return Result.error("失败");
    }

    @Override
    public Result<EngineMaintenance> insert(EngineMaintenance engineMaintenance) {
        Integer res = engineMapper.insert(engineMaintenance);
        if (res > 0) {
            return Result.success();
        }
        return Result.error("失败");
    }

    @Override
    public EngineVersionListResp getRuleEngineInfo(Long id) {
        EngineMaintenance engineMaintenance = this.getOne(new LambdaQueryWrapper<EngineMaintenance>() {{
            eq(EngineMaintenance::getId, id);
        }});
        List<EngineVersion> engineConfigs = engineConfigService.list();
        List<EngineVersion> engineConfigList = engineConfigs.stream().filter(engineConfig ->
                engineConfig.getEngineMaintenanceId().equals(engineMaintenance.getId())
        ).toList();
        return EngineVersionListResp.builder()
                .id(engineMaintenance.getId())
                .name(engineMaintenance.getName())
                .type(engineMaintenance.getType())
                .scope(Integer.valueOf(engineMaintenance.getScope()))
                .engineCode(engineMaintenance.getEngineCode())
                .isActivate(engineMaintenance.getIsActivate())
                .status(engineMaintenance.getStatus())
                .description(engineMaintenance.getDescription())
                .engineVersionList(engineConfigList)
                .build();
    }


    @Override
    public void onEngine(Integer id) {
        Integer res = engineMapper.onEngine(id);
    }

    @Override
    public void closeEngine(Integer id) {
        Integer res = engineMapper.closeEngine(id);
    }

    @Override
    public void activateEngine(Integer id) {
        engineMapper.activateEngine(id);
    }

    @Override
    public int deleteByIds(String ids) {
        LambdaQueryWrapper<EngineMaintenance> engineMaintenanceLambdaQueryWrapper = new LambdaQueryWrapper<>();
        engineMaintenanceLambdaQueryWrapper.in(EngineMaintenance::getId, ids);
        int delete = engineMapper.delete(engineMaintenanceLambdaQueryWrapper);
        return delete;
    }

    @Override
    public Result<EngineMaintenance> add(EngineMaintenance engineMaintenance) {
        Integer res = engineMapper.add(engineMaintenance);
        if (res > 0) {
            return Result.success();
        }
        return Result.error();
    }

    @Override
    public Result<PageResult<EngineMaintenance>> getMaintenanceList(EngineMaintenanceQueryReq engineMaintenanceQueryReq) {
        PageHelper.startPage(engineMaintenanceQueryReq.getPageNum(), engineMaintenanceQueryReq.getPageSize());
        List<EngineMaintenance> engineMaintenances = engineMapper.getMaintenanceList(engineMaintenanceQueryReq);
        PageInfo<EngineMaintenance> engineMaintenancePageInfo = new PageInfo<>(engineMaintenances);
        return Result.success(PageResult.toPageResult(engineMaintenancePageInfo.getTotal(), engineMaintenances));
    }

    @Override
    public Integer deletes(Long id) {
        Integer res = engineMapper.deletes(id);
        return res;
    }

    @Override
    public Result forbiddenEngine(Integer id) {
        Integer res = engineMapper.forbiddenEngine(id);
        return Result.success(res);
    }

    @Override
    public EngineMaintenance selectById(Long id) {
        EngineMaintenance engineMaintenance = engineMapper.select(id);
        return engineMaintenance;
    }
}
